#packages
#using NBInclude
#
# using Distributions
# using Combinatorics
# using DelimitedFiles
# using DataFrames
using JuMP
# using CSV
# using StatsBase
# using StatsPlots
using CPLEX
using JLD2
using CSV
using DataFrames

# using CouenneNL
# using Juniper
# using NLopt
# using AmplNLWriter

ATAmodel=StartATA() #for resetting the ATA process (Needed)
ATAmodel=LoadSettings(ATAmodel;settingsFile="settingsATA.jl",BankFile="estPool.csv", BankDelim=",")  #add file with custom settings (Needed)
ATAmodel=AddCatConstr(ATAmodel;file="Categorical Constraints.csv", delim=";")
ATAmodel=AddOverlaps(ATAmodel; file="OverlapMatrix.csv") #add overlap maxima (Optional)
ATAmodel=AddFriendSets(ATAmodel)
ATAmodel.Settings.OptType="CC"
AddObjFun(ATAmodel, folder="BS", AuxInt=500, AuxFloat=0.05) #add objective function (Needed)
ATAmodel.Settings.OptType="MAXIMIN"
#ATAmodel=AddExpScore(ATAmodel) #add expected score cosnstraints (Optional)
#Assemble!
#All the settings and outputs from optimization are in ATAmodel.

#MAXIMIN
JLD2.@load "OPT/IIF.jld2" IIF

#OVERLAP
using Combinatorics
opMatrix=Matrix(CSV.read("overlapMatrix.csv",delim=';',header=false))
Pairs=collect(combinations(1:ATAmodel.Settings.T,2))
nPairs=size(Pairs,1)
ol_max=Array{Int64,1}(undef,nPairs)
fInd=[Pairs[pair][1] for pair in 1:nPairs]
fIndFirst=[Pairs[pair][2] for pair in 1:nPairs]
for pair in 1:nPairs
	ol_max[pair]=opMatrix[fInd[pair],fIndFirst[pair]]
end


m=Model(solver=CplexSolver(CPX_PARAM_TILIM=1000))
global c=1
	#decision variables
	@variable(m, x[i=1:ATAmodel.Settings.nItems,t=1:ATAmodel.Settings.T] , Bin)
	#cons vars
	#overlap vars
	@variable(m, y[i=1:ATAmodel.Settings.nItems,p=1:nPairs], Bin)

#objective bound
@variable(m, w >=0)
#objective max info at theta mean
#MAXIMIN
for t=1:ATAmodel.Settings.T
		for k=1:size(ATAmodel.Obj.OptPts[t],1)
			@constraint(m, sum(round(IIF[t][k,i];digits=4)*x[i,t] for i=1:ATAmodel.Settings.nItems) >=w)
		end
end
#add quals and quants constraints

global c=1

#length
for t=1:ATAmodel.Settings.T
		if ATAmodel.Constraints[t].length_max>0
			@constraint(m,  sum(x[i,t] for i=1:ATAmodel.Settings.nItems) - ATAmodel.Constraints[t].length_max<=0)
		end
		if ATAmodel.Constraints[t].length_min>0
			@constraint(m,  -sum(x[i,t] for i=1:ATAmodel.Settings.nItems) + ATAmodel.Constraints[t].length_min<=0)
		end
end

# #item use
for i in 1:ATAmodel.Settings.nItems
	if ATAmodel.IU.Min[i] .> 0
		@constraint(m,  ATAmodel.IU.Min[i]-sum(x[i,t] for t=1:ATAmodel.Settings.T) <= 0 )
		global c+=1
	end
	@constraint(m,  sum(x[i,t] for t=1:ATAmodel.Settings.T) - ATAmodel.IU.Max[i] <= 0)
	global c+=1
end

#overlap classic
for p=1:nPairs
@constraint(m, sum(y[i,p] for i=1:ATAmodel.Settings.nItems)-ol_max[p]<=0)
global c+=1
@constraint(m,  [i = 1:ATAmodel.Settings.nItems], 2*y[i,p]<=x[i,fInd[p]]+x[i,fIndFirst[p]])
@constraint(m,  [i = 1:ATAmodel.Settings.nItems], y[i,p]>=x[i,fInd[p]]+x[i,fIndFirst[p]]-1)
end
#expected score
for t=1:ATAmodel.Settings.T
	if size(ATAmodel.Constraints[t].ExS.Val,1)>0
		@constraint(m, sum(x[i,t]*ATAmodel.Constraints[t].ExS.Val[i] for i=1:nItems)<=ATAmodel.Cosntraints[t].ExS.Max*ATAmodel.Constraints[t].lenght_min+0)
		global c+=1
		@constraint(m, sum(x[i,t]*ATAmodel.Constraints[t].ExS.Val[i] for i=1:nItems)>=ATAmodel.Cosntraints[t].ExS.Min*ATAmodel.Constraints[t].lenght_max-0)
		global c+=1
	end
end
#Friend Sets
if size(ATAmodel.Settings.FS.Items,1)!=ATAmodel.Settings.nItems
for t=1:ATAmodel.Settings.T
	for fs=1:size(ATAmodel.Settings.FS.Items,1)
		sizeFS=size(ATAmodel.Settings.FS.Items[fs],1)-1
		if sizeFS>0
			oneoversizeFS=1/sizeFS
			@constraint(m, sum(oneoversizeFS.*x[i,t] for i in ATAmodel.Settings.FS.Items[fs][1:(end-1)])-sizeFS*x[ATAmodel.Settings.FS.Items[fs][end],t]<=0)
			global c+=1
			@constraint(m, -sum(oneoversizeFS.*x[i,t] for i in ATAmodel.Settings.FS.Items[fs][1:(end-1)])+sizeFS*x[ATAmodel.Settings.FS.Items[fs][end],t]>=0)
			global c+=1
		end
	end
end
end
#enemysets
if size(ATAmodel.Settings.ES.Sets,1)>0
for t=1:ATAmodel.Settings.T
	for es=1:size(ATAmodel.Settings.ES.Sets,1)
		@constraint(m, sum(x[i,t] for i in ATAmodel.Settings.ES.Items[es])<=1+0)
		global c+=1
	end
end
end
#cosntraints
for t=1:ATAmodel.Settings.T
	if size(ATAmodel.Constraints[t].catConstrA,1)>0
		for constr=1:size(ATAmodel.Constraints[t].catConstrA,1)
			@constraint(m, sum(x[i,t]*ATAmodel.Constraints[t].catConstrA[constr,i] for i=1:ATAmodel.Settings.nItems)<=ATAmodel.Constraints[t].catConstrb[constr]+0)
			global c+=1
		end
	end
end
global ncons=copy(c)-1
@objective(m, Min, -0.9*w)

status=solve(m)
global design=getvalue(x)
global y_design=getvalue(y)
writedlm("RESULTS/design.csv",design)
#end #endloop
ATAmodel.Output.Design=abs.(vec(design))
ATAmodel.Settings.OptType="CC"

PrintResults(ATAmodel;GroupByFS=false, BSfolder="BS") #BSfolder is for CCATA, save all the plots in RESULTS folder
